/**
 * Created by litao on 2018/9/29.
 */

import Vue from 'vue';
import Utils from '../../utils/index';
const _import = require('./import/_import_' + process.env.NODE_ENV);

class Base {
    constructor (){
        try {
            require.context('sourcePath/', false, /Root\.vue/);
        } catch (e) {
            throw new Error('请在webpack中配置[resolve.alias]，key为[sourcePath]，值为当前项目的资源存放路径，入文件都在src目录下，则值为src目录的绝对路径');
        }

        //初始化配置
        this.initConfigure();

        //暴露vue，不用多次引用
        this.initProperty('$vue', Vue);
    }

    /**
     * 初始化配置
     */
    initConfigure (){
        this.initProperty('$configureFile', new Map());
        let configure = require.context('sourcePath/', true, /\.ezv\.js$/);
        configure.keys().forEach(file => {
            let pathArr = file.split('/').filter(item => item);
            if(pathArr.length > 0){
                let configureName = pathArr[pathArr.length - 1].replace('\.ezv\.js', '');

                let instance = configure(file).default;

                Utils.assert(instance && Utils.equalsType(instance, Function), `所有配置必须以函数形式返回，【${configureName}】的配置文件返回为空或者不是一个函数`);

                this.$configureFile.set(configureName, {
                    path: file,
                    instance
                });
            }
        });
    }

    /**
     * 获取配置
     * @param name
     * @param type
     * @param params
     * @return {*|{code, label}}
     */
    getConfigure (name, type, params){
        let nameArray = name.split('\.');
        let configureName = nameArray[0];

        let configure = this.$configureFile.get(configureName);

        if(!configure) return;

        let obj;
        if(nameArray.length > 1){ //如果是多级配置文件，则判断有参数就执行，没参数就不当做方法执行
            let configureInstance = configure.instance();

            handler(nameArray.splice(1));

            function handler(source) {
                source.forEach(item => {
                    obj = configureInstance[item];
                });
            }

            if(params){
                obj = obj(params);
            }
        }else{ //如果是一级配置文件，则直接当做方法执行
            obj = configure.instance(params);

            Utils.assert(obj && (!!type || Utils.equalsType(obj, type)), `${name}配置的返回值不能为空，且必须为${type.name}`);
        }

        return obj;
    }

    /**
     * 文件加载处理
     * @param loadPath
     * @return {*}
     */
    fileLoadFunc (loadPath){
        Utils.assert(loadPath, '加载路由页面时，未传入路径');
        return _import(loadPath);
    }

    /**
     * 初始化属性
     * @param key
     * @param value
     */
    initProperty (key, value){
        Utils.setProperty(this, key, value);
    }
}

export default Base;