/**
 * Created by litao on 2018/10/4.
 */

let JBootInstanceModuleId = 0;

import Base from './base/index';
import List from './list/index';
import Form from './form/index';
import Utils from '../../../utils/index';

class JBootInstance {
    constructor (options, container){
        //继承关系
        this._extend = ['base'];

        //系统内置模块
        this.$systemModules = ['list', 'form'];

        //忽略转换的模块
        this.$ignoreProperty = ['beforeCreate', 'created', 'beforeMount', 'mounted', 'beforeUpdate', 'updated', 'activated', 'deactivated', 'beforeDestroy', 'destroyed'];

        //复制属性
        this.copyOptions(options, container);

        //扩展设置module
        this.extendModule();
    }

    /**
     * 初始化参数
     * @param options
     * @param containerInstance
     */
    copyOptions (options, containerInstance){
        this.$container = containerInstance;
        this.options = Object.assign({}, options);
    }

    /**
     * 扩展module设置
     */
    extendModule (){
        Utils.equalsType(Object.prototype.module, undefined) && Object.defineProperty(Object.prototype, 'module' ,{
            value: function setCurrentExtendModule() {
                for(let i=0; i<arguments.length; i++){
                    let $module = arguments[i];

                    if(!$module) continue;

                    if(i > 0) {
                        this._extend.push($module);
                        continue;
                    }

                    if(this._extend){
                        let objType = Object.prototype.toString.call(this._extend).toLowerCase();
                        if(objType === '[object string]'){
                            this._extend = [$module];
                        }else if(objType === '[object array]'){
                            this._extend.push($module);
                        }
                    }else{
                        this._extend = [$module];
                    }
                }

                return this;
            }
        });
    }

    /**
     * 获取合并方法
     * @return {*}
     */
    get mergeFn (){
        return this.$container.$vue.config.optionMergeStrategies;
    }

    /**
     * 获取合并的对象
     * @return {*}
     */
    get mergeOptions (){
        return Object.assign(this.$container.$wrap || {}, {
            base: Base,
            list: List,
            form: Form
        });
    }

    list (){
        this._extend.push('list');
        return this;
    }

    form (){
        this._extend.push('form');
        return this;
    }

    store (...args){
        this._stores = args || [];
        return this;
    }

    /**
     * 转换至vue对象
     */
    transform2VueObject (){
        const _self = this;

        this.options.mixins = this.options.mixins || [];
        this.options.mixins.push({
            beforeCreate (){
                _self._extend.forEach(mergeOptionsKey => _self.mergeOptionsFn(mergeOptionsKey, this));

                Utils.equalsType(_self._stores, Array) && _self._stores.forEach(store => {
                    let stores = store.split('/').filter(item => item);

                    let params = { computed: {} };

                    params.computed[stores.join('_')] = function () {
                        let obj = this.$store.getters;
                        stores.forEach(key => {
                            obj = obj[key];
                            if(Utils.equalsType(obj, undefined)) return false;
                        });

                        return obj;
                    };

                    _self.mergeMixinFn('computed', params, this);
                });

                Object.assign(this, {
                    _isJBootInstance: true,
                    _JBootInstance: {
                        moduleId: JBootInstanceModuleId++,
                        container: _self.$container
                    }
                });
            }
        });

        return this.options;
    }

    /**
     * 合并处理
     * @param mergeOptionsKey
     * @param vm
     */
    mergeOptionsFn (mergeOptionsKey, vm){
        let options = this.mergeOptions[mergeOptionsKey];

        if(!Utils.equalsType(options, Object)){
            _console.warn(`未找到需要继承的父类[${mergeOptionsKey}]`);
            return;
        }

        Object.keys(options).forEach(property => {
            //继承关系中存在其他继承
            if(['$module'].includes(property)){
                this.mergeOptionsFn(options[property], vm);
            }else{
                this.mergeMixinFn(property, options, vm);
            }
        });
    }

    /**
     * 合并mixin处理
     * @param property
     * @param options
     * @param vm
     */
    mergeMixinFn (property, options, vm){
        let mergeFunc = this.mergeFn[property];
        if(Utils.equalsType(mergeFunc, Function)){
            let mixinObj = options[property];
            if(['props'].concat(this.$ignoreProperty).includes(property)){
                if(!Utils.isArray(mixinObj)){
                    mixinObj = [mixinObj];
                }
            }
            vm.$options[property] = mergeFunc(mixinObj, vm.$options[property], vm, property);
        }
    }

    /**
     * 执行编译
     * @return {*}
     */
    build (){
        return this.transform2VueObject();
    }
}

export default JBootInstance;