export default {
    dateFormat(value, format = 'yyyy-MM-dd hh:mm:ss', sofar = false) {
        if (value) {
            let result = new Date(value).format(format);
            if (result === '1970-01-01' && sofar || result === '1970.01.01' && sofar) {
                return '至今';
            }
            return result
        }
    },

    toThousandslsFilter(num) {
        return (+num || 0).toString().replace(/^-?\d+/g, m => m.replace(/(?=(?!\b)(\d{3})+$)/g, ','))
    },

    numberFormatter(num, digits) {
        const si = [
            {value: 1E18, symbol: 'E'},
            {value: 1E15, symbol: 'P'},
            {value: 1E12, symbol: 'T'},
            {value: 1E9, symbol: 'G'},
            {value: 1E6, symbol: 'M'},
            {value: 1E3, symbol: 'k'}
        ];
        for (let i = 0; i < si.length; i++) {
            if (num >= si[i].value) {
                return (num / si[i].value + 0.1).toFixed(digits).replace(/\.0+$|(\.[0-9]*[1-9])0+$/, '$1') + si[i].symbol
            }
        }
        return num.toString()
    },

    timeAgo(time) {
        const between = Date.now() / 1000 - Number(time)
        if (between < 3600) {
            return pluralize(~~(between / 60), ' minute')
        } else if (between < 86400) {
            return pluralize(~~(between / 3600), ' hour')
        } else {
            return pluralize(~~(between / 86400), ' day')
        }
    },

    pluralize(time, label) {
        if (time === 1) {
            return time + label
        }
        return time + label + 's'
    }
}
