/**
 * Created by litao on 2018/10/18.
 */
import Vue from 'vue';

//message容器实例
let messageContainerInstance;

/**
 * 获取message容器
 */
function getMessageContainer() {
    return messageContainerInstance = messageContainerInstance || createMessageContainer();
}

/**
 * 创建message容器
 */
function createMessageContainer() {
    return new Vue({
        render (h){
            return h('div', {
                class: 'jboot-message-container',
                style: {
                    zIndex: window.__JBootInstance__ ? window.__JBootInstance__.$EZV_zIndex : 3001
                }
            });
        }
    }).$mount();
}

/**
 * 创建消息提示项
 * @param animateOpt
 * @param content
 * @param duration
 * @return {*}
 */
function createMessageItem(animateOpt, content, duration) {
    return new Vue({
        render (h){
            return h('transition', {
                props: {
                    enterActiveClass: animateOpt.inClass,
                    leaveActiveClass: animateOpt.outClass
                }
            }, [
                this.show && h('div', {
                    class: 'jboot-message-item'
                }, [
                    h('div', {
                        class: `jboot-message-item-instance ${animateOpt.class}`,
                        on: {
                            mouseover: this.stopTimeOut,

                            mouseout: () => {
                                if(!this.show) return;

                                this.startTimeOut();
                            }
                        }
                    }, [
                        h('ev-iconFont', {
                            props: {
                                icon: animateOpt.icon
                            }
                        }),
                        h('span', {
                            attrs: {
                                title: content
                            }
                        }, content),
                        h('ev-iconFont', {
                            class: 'message-close-icon',
                            props: {
                                icon: 'icon-cuo'
                            },
                            nativeOn: {
                                click: () => this.show = false
                            }
                        })
                    ])
                ])
            ]);
        },

        data (){
            return {
                show: false,
            }
        },

        mounted (){
            this.startTimeOut();
        },

        methods: {
            startTimeOut (){
                this.stopTimeOut();
                this.show = true;
                this.timeOutId = setTimeout(() => {
                    this.show = false;
                }, duration);
            },

            stopTimeOut (){
                if (this.timeOutId)
                    clearTimeout(this.timeOutId);
            },

            consoleInfo (type){
                if(!window.__JBootInstance__) return;

                let route = window.__JBootInstance__.$route;

                if(route && window.__JBootInstance__.$getObjectType(route) === '[object object]'){
                    let errorSign;

                    if(route.meta && route.meta.title){
                        errorSign = route.meta.title;
                    }else if(route.name){
                        errorSign = route.name;
                    }

                    if(errorSign){

                        try{
                            throw new Error();
                        } catch (e) {
                            if(type === 'error'){
                                _console.error(e.stack.replace('Error', `[${errorSign}] 抛出了一个异常信息，信息内容为：${msg}，堆栈信息如下：`));
                            }else{

                                _console.warn(e.stack.replace('Error', `[${errorSign}] 抛出了一个警告信息，信息内容为：${msg}，堆栈信息如下：`));
                            }
                        }
                    }
                }
            }
        }
    }).$mount();
}

export default {
    create (animateOpt, content, duration = 3000){
        let instance = getMessageContainer();

        let messageItem = createMessageItem(animateOpt, content, duration);

        instance.$el.appendChild(messageItem.$el);

        document.body.appendChild(instance.$el);
    }
}