/**
 * Created by litao on 2018/9/29.
 */

import './style/animate.css';
import './style/system-component.scss';
import './style/iconfont/iconfont.css';
import Container, { responseCheckSuccessHook } from './container';
import Utils from './utils/index';

/**
 * 基础实例
 */
class JBoot{
    /**
     * 构造函数
     * @param options
     */
    constructor (options = {}){
        let { Root, el = '#app', log = true, plugins, tokenKey = 'JBootDefaultTokenKey', responseStatusCheck } = options;
        initLogWrap(log);

        //初始化容器
        Utils.setProperty(this, '$container', new Container());

        //初始化tokenKey值
        this.$container.$vue.prototype.$token.setTokenKey(tokenKey, this.$container.$vue);

        //初始化处理http返回状态函数
        if(Utils.equalsType(responseStatusCheck, Function)){
            responseCheckSuccessHook(responseStatusCheck);
        }

        //插件安装
        if(plugins){
            if(Utils.equalsType(plugins, Array)){
                plugins.forEach(plugin => this.$container.$vue.use(plugin));
            }else{
                this.$container.$vue.use(plugins);
            }
        }

        if(!Root){
            let ctx = require.context('sourcePath/', false, /Root\.vue/);

            ctx.keys().forEach(path => {
                if(Root) return;

                Root = ctx(path).default;
            });
        }else{
            Utils.assert(Root.__file && Root.__file.endsWith('\.vue'), '实例化JBoot时，根实例文件必须为.vue后缀的文件');
        }

        Utils.assert(Root, '实例化JBoot时，未获取到根实例文件[Root.vue]');

        try {
            document.querySelector(el);
        } catch (e) {
            Utils.assert(false, `实例化JBoot时，未找到el标识为[${el}]的dom节点`);
        }

        //存储el标识
        Utils.setProperty(this, '$el', el);

        //初始化实例
        const instance = this.$container.createVueInstance(el, Root);

        Utils.setProperty(this, '$instance', instance);

        window.__JBootInstance__ = instance;
    }
}

JBoot.LOG_LEVEL = {
    ALL: 4,
    INFO: 3,
    WARN: 2,
    ERROR: 1,
    CLOSE: 0
};

function initLogWrap(flag) {
    const logLevelMap = ['log', 'info', 'warn', 'error'].reverse();
    window._console = {};
    for(let i=0; i<logLevelMap.length; i++){
        let level = logLevelMap[i];
        window._console[level] = function () {
            flag && window.console[level].apply(this, arguments);
        };
    }
}

export default JBoot;